/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.editpart.policy;

import org.eclipse.draw2d.Viewport;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.Request;
import org.eclipse.gef.editparts.ScalableRootEditPart;
import org.eclipse.gef.editpolicies.NonResizableEditPolicy;
import org.eclipse.gef.requests.ChangeBoundsRequest;
import org.fortiss.tooling.base.model.layout.Dimension;
import org.fortiss.tooling.base.model.layout.ILayoutedModelElement;
import org.fortiss.tooling.base.model.layout.OffsetOrientation;
import org.fortiss.tooling.base.ui.editpart.ElementEditPartBase;
import org.fortiss.tooling.base.ui.layout.constants.DefaultLayoutConstants;
import org.fortiss.tooling.base.ui.layout.constants.LayoutKeyConstants;
import org.fortiss.tooling.base.ui.layout.util.LayoutDataUtils;
import org.fortiss.tooling.base.ui.layout.util.RectangleLayoutUtils;
import org.fortiss.tooling.base.ui.layout.util.SnapToGridUtils;
import org.fortiss.tooling.kernel.model.INamedCommentedElement;

/**
 * EditPolicy for keeping a connector sticking to the border of it's parent
 * element.
 * <p>
 * This EditPolicy is responsible for limiting connector placement to the border
 * of a model element. It's clear, that this is only valid for sticky
 * connectors, all others are not affected by this policy.
 * 
 * @author wandinger
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: D8173BE506F1A36E8BF28B17E1BD9C1E
 */
public class ConstrainedNonResizableEditPolicy<T extends ILayoutedModelElement & INamedCommentedElement>
		extends NonResizableEditPolicy {

	/** Stores the edit part. */
	private final ElementEditPartBase<T> part;

	/** Constructor. */
	public ConstrainedNonResizableEditPolicy(ElementEditPartBase<T> part) {
		this.part = part;
	}

	/** {@inheritDoc} */
	@SuppressWarnings("unchecked")
	@Override
	public void showSourceFeedback(Request request) {
		if (REQ_MOVE.equals(request.getType())) {
			// get the bounds of the parent
			Rectangle containerBounds = LayoutDataUtils
					.getNodeBounds((ILayoutedModelElement) part.getModel());
			// get the child position relative to the parent
			Point childPos = getHostFigure().getBounds().getLocation()
					.getTranslated(containerBounds.getLocation().getNegated());
			// get the current mouse position
			Point mousePos = ((ChangeBoundsRequest) request).getLocation();
			// fix offset in mouse position because of possibly scrolled editing
			// window
			Point clientAreaOffset = new Point(
					((Viewport) ((ScalableRootEditPart) part.getRoot())
							.getFigure()).getClientArea().getLocation());
			mousePos.translate(clientAreaOffset);
			// get the mouse position relative to the parent
			Point relativePos = mousePos.getTranslated(containerBounds
					.getLocation().getNegated());

			// do border clamping
			OffsetOrientation orient = RectangleLayoutUtils
					.calculateRectangleBorderOrientation(relativePos,
							containerBounds.getSize(),
							DefaultLayoutConstants.DEFAULT_SHAPE_INSETS);
			// snap to grid
			orient.setOffset(SnapToGridUtils.snapToGrid(orient.getOffset()));
			// use the calculated value for positioning
			Dimension d = LayoutDataUtils
					.getNodeSize((ILayoutedModelElement) part.getModel());
			org.fortiss.tooling.base.model.layout.Point p = LayoutDataUtils
					.getAbsolute(orient, d,
							DefaultLayoutConstants.DEFAULT_SHAPE_INSETS);
			relativePos.setLocation(p.getX(), p.getY());

			// store the orientation in the request so that the command can use
			// it
			request.getExtendedData().put(
					LayoutKeyConstants.CONNECTOR_OFFSET_ORIENTATION, orient);
			// MoveDelta needs positions relative to the child
			relativePos.translate(childPos.getNegated());

			((ChangeBoundsRequest) request).setMoveDelta(relativePos);
		}
		super.showSourceFeedback(request);
	}
}
