/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2011 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.extension.data;

import java.util.Comparator;

import org.eclipse.emf.ecore.EObject;

/**
 * This interface describes a constraint violation produced by some constraint
 * checker. A constraint violation has a severity of type {@link ESeverity}, an
 * explanation of type String and possibly a quick fix handler of type
 * {@link IQuickFixHandler}.
 * 
 * @author hoelzlf
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating GREEN Hash: 59B63AA8B22AF7224E17B92B0CD261BF
 */
public interface IConstraintViolation<T extends EObject> {

	/** Returns the source of the constraint violation. */
	T getSource();

	/** Returns the severity of the constraint violation. */
	ESeverity getSeverity();

	/** Returns the explanation of the constraint violation. */
	String getExplanation();

	/** Returns the quick fix handler for this constraint violation. */
	IQuickFixHandler getQuickFixHandler();

	/** Enumeration of severity classes. */
	public static enum ESeverity {
		/** Fatal error severity. */
		FATAL,
		/** Normal error severity. */
		ERROR,
		/** Warning severity. */
		WARNING,
		/** Information severity. */
		INFO,
		/** Debug severity. */
		DEBUG;

		/** Translates ESeverity to int. */
		public static int getIntSeverity(ESeverity severity) {
			switch (severity) {
			case FATAL:
				return -5;
			case ERROR:
				return -3;
			case WARNING:
				return -1;
			case INFO:
				return 1;
			}
			// ESeverity.DEBUG
			return 5;
		}

		/** Returns the lowest severity level. */
		public static ESeverity lowest() {
			return DEBUG;
		}
	}

	/** Interface for quick fix handlers. */
	public static interface IQuickFixHandler {
		/** Applies the quick fix. */
		void applyQuickFix();
	}

	/** Stores the severity comparator used during violation sorting. */
	public static final Comparator<IConstraintViolation<EObject>> SEVERITY_COMPARATOR = new Comparator<IConstraintViolation<EObject>>() {
		/** {@inheritDoc} */
		@Override
		public int compare(IConstraintViolation<EObject> arg0,
				IConstraintViolation<EObject> arg1) {
			int sev0 = ESeverity.getIntSeverity(arg0.getSeverity());
			int sev1 = ESeverity.getIntSeverity(arg1.getSeverity());
			return sev0 - sev1;
		}
	};
}
